// $Id: popups.js,v 1.9.2.4 2008/08/28 03:27:37 starbow Exp $

/**
 * Popup Modal Dialog API
 *
 * Provide an API for building and displaying JavaScript, in-page, popups modal dialogs.
 * Modality is provided by a fixed, semi-opaque div, positioned in front of the page contents.
 *
 */

/**
 * Create the popups object/namespace.
 */
Drupal.popups = function() {};

/**
 * Attach the popups bevior to the all the requested links on the page.
 *
 * @param context
 *   The jQuery object to apply the behaviors to.
 */
Drupal.behaviors.popups = function(context) {
//console.log("Attaching popups");
  var $body = $('body');
  if(!$body.hasClass('popups-processed')) {
    $(document).bind('keydown', Drupal.popups.keyHandle);
    $body.addClass('popups-processed');
  }
  
  // Add the popups-link-in-dialog behavior to links defined in Drupal.settings.popups.links array.
  if (Drupal.settings.popups) {
    for (var link in Drupal.settings.popups.links) {
      var options = Drupal.settings.popups.links[link];
      Drupal.popups.attach(context, link, options); // Needs to be seperate function for closure.
    }
  }
  
  Drupal.popups.attach(context, '.popups', {});  
  Drupal.popups.attach(context, '.popups-form', {updatePage: true});  
};

/**
 * Attach the popups behavior to a particular link.
 *
 * @param selector
 *   jQuery selector for links to attach popups behavior to.
 * @param options
 *   Hash of options associated with these links.
 */
Drupal.popups.attach = function(context, selector, options) {
  $(selector, context).not('.popups-processed').each( function() {
    var $element = $(this);
    
    $element.click(function(e){ 
      var element = this;

      // If the element contains a on-popups-options attribute, use it instead of options param.
      if ($(element).attr('on-popups-options')) {
        eval('options = ' + $(element).attr('on-popups-options'));
      }

      // If the option is distructive, check if the page is already modified, and offer to save.
      var pageIsDirty = $('span.tabledrag-changed').size() > 0;
      var willModifyOriginal = !options.noReload;
//    console.log ( "LOG: pageIsDirty && willModifyOriginal:" + pageIsDirty + ', '+ willModifyOriginal);
      if (pageIsDirty && willModifyOriginal) {
        // The user will lose modifications, so popups dialog offering to save current state.
        var body = Drupal.t("There are unsaved changes on this page, which you will lose if you continue.");
        var buttons = {
         'popup_save': {title: Drupal.t('Save Changes'), func: function(){Drupal.popups.savePage(element, options);}},
         'popup_submit': {title: Drupal.t('Continue'), func: function(){Drupal.popups.removePopup(); Drupal.popups.openPath(element, options);}},
         'popup_cancel': {title: Drupal.t('Cancel'), func: Drupal.popups.close}
        };
        return Drupal.popups.open( Drupal.t('Warning: Please Confirm'), body, buttons );
      }
      else {
        return Drupal.popups.openPath(element, options);
      } 
    });
    var title = $element.attr('title') || '';
    $element.attr('title', title + Drupal.t('[Popup]')); // Append note to link title.
    $element.addClass('popups-processed');
  });
};


/**
 * Generic dialog builder.
 */
Drupal.popups.open = function(title, body, buttons, width) {

  Drupal.popups.addOverlay(); // TODO - nonModal option.
  var $popups = $(Drupal.theme('popupDialog', title, body, buttons));
  // Start with dialog off the side. Making it invisible causes flash in FF2.
  $popups.css('left', '-9999px');
  if (width) {
    $popups.css('width', width);
  }
  $('body').append( $popups ); // Add the popups to the DOM.

  // Adding button functions
  if (buttons) {
    for (var id in buttons) {
      if (buttons[id]) { // to make jslint happy.
        var func = buttons[id].func;
        $('#'+id).click( func );
      }
    }
  }
  $('#popups-close').click( Drupal.popups.close );
  $('a.popups-close').click( Drupal.popups.close );
    
  // center on the screen, adding in offsets if the window has been scrolled
  var popupWidth = $popups.width();  
  var windowWidth = $(window).width();
  var left = (windowWidth / 2) - (popupWidth / 2) + Drupal.popups.scrollLeft();
  
  // Get popups's height on the page.
  // Causes flash in FF2 if popups is not visible!
  var popupHeight = $popups.height(); 
  var windowHeight = $(window).height();
  if (popupHeight > (0.9 * windowHeight) ) { // Must fit in 90% of window.
    popupHeight = 0.9 * windowHeight;
    $popups.height(popupHeight);
  }  
  var top = (windowHeight / 2) - (popupHeight / 2) + Drupal.popups.scrollTop();

  $popups.css('top', top).css('left', left); // Position the popups to be visible.
  
  this.refocus(); // TODO: capture the focus when it leaves the dialog.
  Drupal.popups.removeLoading(); // Remove the loading img.
   
  return false;
};

/**
 *  Simple popups that functions like the browser's alert box.
 */
Drupal.popups.message = function(message, body) {
  body = body || '';
  var buttons = {
    'popup_ok': { title: Drupal.t('OK'), func: Drupal.popups.close }
  };
  Drupal.popups.open( message, body, buttons );
};

/**
 * Handle any special keys when popups is active.
 */
Drupal.popups.keyHandle = function(e) {
  if (!e) {
    e = window.event;
  }
  switch (e.keyCode) {
    case 27: // esc
      Drupal.popups.close();
      break;
    case 191: // '?' key, show help.
      if (e.shiftKey && e.ctrlKey) {
        var $help = $('a.popups.more-help');
        if ($help.size()) {
          $help.click();
        }
        else {
          Drupal.popups.message(Drupal.t("Sorry, there is no additional help for this page"));
        }
      }
      break;
  }
};

/*****************************************************************************
 * Appearence Functions (overlay, loading graphic, remove popups)     *********
 *****************************************************************************/

Drupal.popups.removePopup = function() {
  $('#popups').remove();  
}; 
 
Drupal.popups.addOverlay = function() {
  var $overlay = $('#popups-overlay');
  if (!$overlay.size()) { // Overlay does not already exist, so create it.
    $overlay = $(Drupal.theme('popupOverlay'));
    $overlay.css('opacity', '0.4'); // for ie6(?)
    $body = $('body');
    // Doing absolute positioning, so make overlay's size equal the entire body.
    $overlay.width($body.width()).height($body.height()); 
    $overlay.click(Drupal.popups.close);
    $body.prepend($overlay);
  }
//  return $overlay;
};

//Drupal.popups.getOverlay = function() {
//  return $('#popups-overlay');
//};

Drupal.popups.removeOverlay = function() {
  $('#popups-overlay').remove();
};

Drupal.popups.addLoading = function() {
  var $loading = $('#popups-loading');
  if (!$loading.size()) { // Overlay does not already exist, so create it.
    var waitImageSize = 100;
    var left = ($(window).width() / 2) - (waitImageSize / 2)  + Drupal.popups.scrollLeft();
    var top = ($(window).height() / 2) - (waitImageSize / 2)  + Drupal.popups.scrollTop();
    $loading = $( Drupal.theme('popupLoading', left, top) );
    $('body').prepend($loading);
  }
};

Drupal.popups.removeLoading = function() {
  $('#popups-loading').remove();
};

/**
 * Remove everything.
 */
Drupal.popups.close = function() {
  Drupal.popups.removePopup();
  Drupal.popups.removeLoading();
  Drupal.popups.removeOverlay();
  return false;
};

/**
 * Set the focus on the popups to the first visible form element, or the first button, or the close link.
 */
Drupal.popups.refocus = function() {
  $focus = $('#popups input:visible:eq(0)');
  if (!focus) {
    $focus = $('#popups-close'); // Doesn't seem to work.
  }
  $focus.focus();
};

/****************************************************************************
 * Theme Functions   ********************************************************
 ****************************************************************************/

Drupal.theme.prototype.popupLoading = function(left, top) {
  var loading = '<div id="popups-loading">';
  loading += '<div style="left:' + left +'px; top:' + top +'px;">';
  loading += '<img src="'+ Drupal.settings.basePath + Drupal.settings.popups.modulePath + '/ajax-loader.gif" />';
  loading += '</div></div>';
  return loading;
};

Drupal.theme.prototype.popupOverlay = function() {
  return '<div id="popups-overlay"></div>';
};

Drupal.theme.prototype.popupButton = function(title, id) {
  return '<input type="button" value="'+ title +'" id="'+ id +'" />';
};

Drupal.theme.prototype.popupDialog = function(title, body, buttons) {
  var template = Drupal.settings.popups.template;
  var popups = template.replace('%title', title).replace('%body', body);
  
  var themedButtons = '';
  for ( var id in buttons) {
    if (buttons[id]) {
      themedButtons += Drupal.theme('popupButton', buttons[id].title, id);
    }
  }
  popups = popups.replace('%buttons', themedButtons);  
  return popups;
};

/****************************************************************************
 * Utility scroll functions taken from:                                   ***
 * http://www.softcomplex.com/docs/get_window_size_and_scrollbar_position.html
 ****************************************************************************/

// FROM jQuery offset
// elem = this[0];
// doc = elem.ownerDocument;
// add(box.left + Math.max(doc.documentElement.scrollLeft, doc.body.scrollLeft),
//     box.top  + Math.max(doc.documentElement.scrollTop,  doc.body.scrollTop));

Drupal.popups.scrollLeft = function() {
  return Math.max(document.documentElement.scrollLeft, document.body.scrollLeft);
//  return Drupal.popups.filterResults (
//    window.pageXOffset ? window.pageXOffset : 0,
//    document.documentElement ? document.documentElement.scrollLeft : 0,
//    document.body ? document.body.scrollLeft : 0 );
};

Drupal.popups.scrollTop = function() {
  return Math.max(document.documentElement.scrollTop, document.body.scrollTop);
//  return Drupal.popups.filterResults (
//    window.pageYOffset ? window.pageYOffset : 0,
//    document.documentElement ? document.documentElement.scrollTop : 0,
//    document.body ? document.body.scrollTop : 0 );
};

//Drupal.popups.filterResults = function(win, docel, body) {
//  var result = win ? win : 0;
//  if (docel && (!result || (result > docel))) {
//    result = docel;
//  }
//  return body && (!result || (result > body)) ? body : result;
//};


/****************************************************************************
 * Page & Form in popups functions                                         ***
 ****************************************************************************/

/**
 * Use Ajax to open the link in a popups window.
 *
 * @param element
 *   Element that was clicked to open the popups.
 * @param options
 *   Hash of options controlling how the popups interacts with the underlying page.
 */
Drupal.popups.openPath = function(element, options) {
  // let the user know something is happening
  $('body').css("cursor", "wait");
  
  // TODO - get nonmodal working.
  if (!options.nonModal) {
    Drupal.popups.addOverlay(); 
  }
  Drupal.popups.addLoading();

  // Set custom headers for all following requests.
  $.ajaxSetup({
    beforeSend: function(xhr) {
      xhr.setRequestHeader("X-Drupal-Render-Mode", 'json/popups');
    }
  });
  
  var href = options.href ? options.href : element.href;
  
  $.getJSON(href, function(json) { 
//    console.log("Drupal.popups.openPath: getJSON"); 
    Drupal.popups.openContent(json.title, json.messages + json.content, options);
    $('body').css("cursor", "auto"); // Return the cursor to normal state.  
  });
         
  return false;         
};

/**
 * Open content in an ajax popups.
 *
 * @param title
 *   String title of the popups.
 * @param content
 *   HTML to show in the popups.
 * @param options
 *   Hash of options controlling how the popups interacts with the underlying page.
 */
Drupal.popups.openContent = function(title, content, options) {
  Drupal.popups.open(title, content, null, options.width); 
  // Add behaviors to content in popups. 
  // TODO: d-n-d: need to click to let go of selection.
  delete Drupal.behaviors.tableHeader; // Work-around for bug in tableheader.js (http://drupal.org/node/234377)
  Drupal.attachBehaviors($('#popups-body'));
  // Adding collapse moves focus.
  Drupal.popups.refocus();


  // If the popups contains a form, capture submits.

  var $form = $('form', '#popups-body');
  $form.ajaxForm({   
    dataType: 'json',   
    beforeSubmit: Drupal.popups.beforeSubmit,
    success: function(response, status) {
      Drupal.popups.formSuccess(response, options);
    }
  });
/*
return;

$(":submit", $form).submit(function() {
    $(this).ajaxSubmit({type: 'post'});
    return false;
}); 
return;

  $(":submit", $form).bind('click', function(e) {
    console.log('ajaxSubmit');
    console.log(this);
    $(this).ajaxSubmit({
      type: 'post',
      dataType: 'json',   
//      beforeSubmit: Drupal.popups.beforeSubmit,
//      success: function(response, status) {
//        Drupal.popups.formSuccess(response, options);
//      }
    });
    return false;
  });
  */
};

/**
 * Do before the form in the popups is submitted.
 */
Drupal.popups.beforeSubmit = function(formData, $form, options) {
  Drupal.popups.removePopup(); // Remove just the dialog, but not the overlay.
  Drupal.popups.addLoading();
//  console.log("Before Submit");
};

/**
 * The form in the popups was successfully submitted
 * Update the originating page.
 * Show any messages in a popups (TODO - make this a configurable option).
 * 
 * @param response
 *   JSON object from server with status of form submission.
 * @param options
 *   Hash of options controlling how the popups interacts with the underlying page.
 *     noReload: bool, does the popups effect the underlying page.
 *     //updatePage: bool, does the popups effect the underlying page.
 *     updateTitle: bool, does the popups change the title of the page.
 *     nonModal: bool, does the popups block access to the underlying page.
 *     targetSelector: jQuery selector, overrides defaultTargetSelector.
 */
Drupal.popups.formSuccess = function(data, options) {
//  if (response.status != 'redirect') { // something went wrong
//    Drupal.popups.message( "Error: " + response.messages + response.content);
//  }
//  else { // Got a good response back from the server.
//    $.getJSON(response.path, function(data) {
  
      // Determine if we are at an end point, or just moving from one popups to another.
      if (!location.href.match(data.path) && !location.href.match(options.forceReturn)) { // Not done yet, so show results in new popups.
        Drupal.popups.removeLoading();
        Drupal.popups.openContent(data.title, data.messages + data.content, options);
      }
      else { // Done, so show messages in dialog and embed the results in the original page.
        if (data.messages) {
          if (Drupal.settings.popups.popupFinalMessage) {
            Drupal.popups.message(data.messages);
          }
          else {
            Drupal.popups.close();
          }
          // Insert the message into the page above the content.
          // Might not be the standard spot, but it is the easiest to find.
          var $next = $(Drupal.settings.popups.defaultTargetSelector);
          $next.parent().find('div.messages').remove(); // Remove the current messages.
          $next.before(data.messages);
        }
            
        // Update the entire content area (defined by 'target selector').
//        if (options.updatePage) { 
        if (!options.noReload) { 
          var target = options.targetSelector;
          if (!target) {
            target = Drupal.settings.popups.defaultTargetSelector;
          }       
          // Update the original page.      
          var $c = $(target).html(data.content); // Inject the new content into the page.
          Drupal.attachBehaviors($c);
        }
        
        // Update the title of the page.
        if (options.updateTitle) {
          $(Drupal.settings.popups.defaultTitleSelector).html(data.title);
          document.title = data.title; // Also update the browser page title (TODO: include site name?).
        }
                
        // Done with changes to the original page, remove effects.
        Drupal.popups.removeLoading();
        if (!data.messages) { 
          // If there is not a messages popups remove the overlay.
          Drupal.popups.removeOverlay();
        }
      }  // End of updating original page.
//    }); // End of good response.
//  }
}; 

/**
 * Submit the page and reload the results, before popping up the real dialog.
 *
 * @param element
 *   Element that was clicked to open the popups.
 * @param options
 *   Hash of options controlling how the popups interacts with the underlying page.
 */
 
Drupal.popups.savePage = function(element, options) {
  var target = Drupal.settings.popups.defaultTargetSelector;
  var $form = $('form', target);
  var ajaxOptions = {
    dataType: 'json',
    beforeSubmit: Drupal.popups.beforeSubmit,   
    beforeSend: function(xhr) {
      xhr.setRequestHeader("X-Drupal-Render-Mode", 'json/popups');
    },
    success: function(response, status) { 
      // Sync up the current page contents with the submit.
      var $c = $(target).html(response.content); // Inject the new content into the page.
      Drupal.attachBehaviors($c);
      // The form has been saved, the page reloaded, now safe to show the link in a popup.
      Drupal.popups.openPath(element, options); 
    } 
  };
  $form.ajaxSubmit(ajaxOptions); // Submit the form. 
};

